function [Stheta]=MatrixLog6(T)
%takes transoformation matrix T in SE(3) and returns corresponding 6-vector of exponential coordinates Stheta

[m,n]=size(T);
R=[T(1,1) T(1,2) T(1,3); T(2,1) T(2,2) T(2,3); T(3,1) T(3,2) T(3,3)];
A=isequal(R,eye(3));
p=[T(1,4); T(2,4); T(3,4)];
if m~=4 || n~=4 || T(4,1)~=0 || T(4,2)~=0 || T(4,3)~=0 || T(4,4)~=1
	display('Input is not a 4x4 transformation matrix')
elseif A~=0
    wskew=zeros(3);
    theta=norm(p);
    v=p/theta;
    w=[wskew(3,2); wskew(1,3); wskew(2,1)];
elseif trace(R)~=-1
    theta=acos((trace(R)-1)/2);
    wskew=(R-R')/(2*sin(theta));
    Gg=eye(3)/theta-wskew/2+wskew^2*(1/theta-1/2*cot(theta/2));
    v=Gg*p;
    w=[wskew(3,2); wskew(1,3); wskew(2,1)];
else
    theta=pi;
    w1=1/sqrt(2*(1+R(3,3)))*[R(1,3);R(2,3);1+R(3,3)];
    w2=1/sqrt(2*(1+R(2,3)))*[R(1,3);1+R(2,3);R(3,3)];
    w3=1/sqrt(2*(1+R(1,3)))*[1+R(1,3);R(2,3);R(3,3)];
    w1=w1(~isnan(w1));
    w2=w2(~isnan(w2));
    w3=w3(~isnan(w3));
    if length(w1)~=3
        if length(w2)~=3
            w=w3;
        else w=w2;
        end
    else w=w1;
    end
    wskew=[0 -w(3) w(2); w(3) 0 -w(1); -w(2) w(1) 0];
    Gg=eye(3)/theta-wskew/2+wskew^2*(1/theta-1/2*cot(theta/2));
    v=Gg*p;
end

S=[w;v];
Stheta=S*theta;
end